﻿using NetinaShop.Domain.Entities.Blogs;

namespace NetinaShop.AdminPanel.PWA.Pages;

public class BlogCategoriesPageViewModel:BaseViewModel<ObservableCollection<BlogCategorySDto>>
{
    private readonly NavigationManager _navigationManager;
    private readonly ISnackbar _snackbar;
    private readonly IUserUtility _userUtility;
    private readonly IDialogService _dialogService;
    private readonly IRestWrapper _restWrapper;

    public string Search = string.Empty;
    public int CurrentPage = 0;
    public int PageCount = 1;

    public BlogCategoriesPageViewModel(NavigationManager navigationManager, ISnackbar snackbar, IUserUtility userUtility, IRestWrapper restWrapper, IDialogService dialogService)
    {
        _navigationManager = navigationManager;
        _snackbar = snackbar;
        _userUtility = userUtility;
        _restWrapper = restWrapper;
        _dialogService = dialogService;
    }

    public override async Task InitializeAsync()
    {
        try
        {
            IsProcessing = true;
            PageDto.Clear();
            var dto = await _restWrapper.CrudDtoApiRest<BlogCategory, BlogCategorySDto, Guid>(Address.BlogCategoryController)
                .ReadAll(CurrentPage);
            dto.ForEach(d => PageDto.Add(d));
            if (PageDto.Count == 20)
                PageCount = 2;
        }
        catch (ApiException ex)
        {
            var exe = await ex.GetContentAsAsync<ApiResult>();
            _snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
        }
        catch (Exception e)
        {
            _snackbar.Add(e.Message, Severity.Error);
        }
        finally
        {

            IsProcessing = false;
        }
        await base.InitializeAsync();
    }

    public async Task ChangePageAsync(int page)
    {
        CurrentPage = page - 1;
        if (CurrentPage > PageCount - 2)
        {

            try
            {
                IsProcessing = true;

                List<BlogCategorySDto> dto = new List<BlogCategorySDto>();
                if (Search.IsNullOrEmpty())
                {
                    dto = await _restWrapper.CrudDtoApiRest<BlogCategory, BlogCategorySDto, Guid>(Address.BlogCategoryController)
                        .ReadAll(CurrentPage);
                }
                else
                {
                    dto = await _restWrapper.BlogCategoryRestApi.ReadAll(CurrentPage, Search);
                }

                dto.ForEach(d => PageDto.Add(d));
                if (PageDto.Count % 20 == 0)
                    PageCount = CurrentPage + 2;

            }
            catch (ApiException ex)
            {
                var exe = await ex.GetContentAsAsync<ApiResult>();
                _snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
            }
            catch (Exception e)
            {
                _snackbar.Add(e.Message, Severity.Error);
            }
            finally
            {

                IsProcessing = false;
            }
        }
    }

    public async Task AddBlogCategoryClicked()
    {
        DialogOptions maxWidth = new DialogOptions() { MaxWidth = MaxWidth.Medium, FullWidth = true, DisableBackdropClick = true };
        await _dialogService.ShowAsync<BlogCategoryActionDialogBox>("افزودن دسته بندی جدید", maxWidth);
    }

    public async Task EditBlogCategoryClicked(BlogCategorySDto blogCategory)
    {
        DialogOptions maxWidth = new DialogOptions() { MaxWidth = MaxWidth.Medium, FullWidth = true, DisableBackdropClick = true };
        var parameters = new DialogParameters<BlogCategoryActionDialogBox>();
        parameters.Add(x => x.BlogCategory, blogCategory);
        await _dialogService.ShowAsync<BlogCategoryActionDialogBox>($"ویرایش دسته بندی {blogCategory.Name}", parameters, maxWidth);
    }

    public async Task DeleteBlogCategoryAsync(Guid selectedCategoryId)
    {
        var options = new DialogOptions { CloseOnEscapeKey = true };
        var parameters = new DialogParameters<QuestionDialog>();
        parameters.Add(x => x.ContentText, "آیا از حذف بلاگ اطمینان دارید ?");
        var dialogReference = await _dialogService.ShowAsync<QuestionDialog>("حذف بلاگ", parameters, options);
        var result = await dialogReference.Result;
        if (!result.Canceled)
        {

            try
            {

                IsProcessing = true;
                var token = await _userUtility.GetBearerTokenAsync();
                await _restWrapper.CrudDtoApiRest<BlogCategory, BlogCategorySDto, Guid>(Address.BlogCategoryController)
                    .Delete(selectedCategoryId, token);
                _snackbar.Add("حذف بلاگ با موفقیت انجام شد", Severity.Success);

            }
            catch (ApiException ex)
            {
                var exe = await ex.GetContentAsAsync<ApiResult>();
                _snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
            }
            catch (Exception e)
            {
                _snackbar.Add(e.Message, Severity.Error);
            }
            finally
            {

                IsProcessing = false;
            }
        }
    }

    public async Task SearchChanged(string search)
    {
        if (search.IsNullOrEmpty() && !Search.IsNullOrEmpty())
            await InitializeAsync();
        Search = search;
    }
    public async Task SearchAsync()
    {
        try
        {
            if (Search.IsNullOrEmpty())
                throw new AppException("نام دسته بندی برای جست جو وارد نشده است");
            IsProcessing = true;
            CurrentPage = 0;
            PageCount = 1;
            PageDto.Clear();
            var dto = await _restWrapper.BlogCategoryRestApi.ReadAll(CurrentPage, Search);
            dto.ForEach(d => PageDto.Add(d));
            if (PageDto.Count == 20)
                PageCount = 2;
        }
        catch (ApiException ex)
        {
            var exe = await ex.GetContentAsAsync<ApiResult>();
            _snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
        }
        catch (Exception e)
        {
            _snackbar.Add(e.Message, Severity.Error);
        }
        finally
        {

            IsProcessing = false;
        }
    }
}